#ifdef LTFAT_COMPLEXTYPE
#  undef LTFAT_COMPLEXTYPE
#endif // LTFAT_COMPLEXTYPE

#include "config.h"
#include "ltfat.h"
#include "ltfat_types.h"


#include "gabdual.c"
#include "gabtight.c"


#define LTFAT_COMPLEXTYPE
#include "ltfat_types.h"

#include "gabdual.c"
#include "gabtight.c"


#undef LTFAT_COMPLEXTYPE


