#ifdef LTFAT_COMPLEXTYPE
#  undef LTFAT_COMPLEXTYPE
#endif // LTFAT_COMPLEXTYPE

#include "ltfat.h"

#include "ltfat_types.h"

#include "goertzel.c"
#include "wavelets.c"
#include "spread.c"
#include "ciutils.c"


#define LTFAT_COMPLEXTYPE
#include "ltfat_types.h"

#include "wavelets.c"
#include "goertzel.c"
#include "spread.c"
#include "ciutils.c"

#undef LTFAT_COMPLEXTYPE


