#ifdef LTFAT_COMPLEXTYPE
#  undef LTFAT_COMPLEXTYPE
#endif // LTFAT_COMPLEXTYPE

#include "ltfat_types.h"
#include "dct.c"

#define LTFAT_COMPLEXTYPE
#include "ltfat_types.h"
#include "dct.c"
#undef LTFAT_COMPLEXTYPE
